import {
    AfterViewInit,
    Component,
    Input,
    OnDestroy,
    OnInit,
    Output,
    TemplateRef,
    ViewChild
} from "@angular/core";
import {
    AbstractControl,
    FormBuilder,
    FormControl,
    FormGroup,
    ValidationErrors,
    ValidatorFn,
    Validators
} from "@angular/forms";
import { ActivatedRoute, Router } from "@angular/router";
import { TranslateService } from "@ngx-translate/core";
import { FieldType, FormField } from "@seniorsistemas/angular-components";
import { Hotkey, HotkeysService } from "angular2-hotkeys";
import * as moment_ from "moment";
import { ConfirmationService } from "primeng/api";
import { MessageService } from "primeng/components/common/messageservice";
import { forkJoin, of, Subject, throwError, Observable } from "rxjs";
import { catchError, map, takeUntil, tap, filter } from "rxjs/operators";
import { EmployeeDate } from "../../../core/dtos/employee-date";

import { Bridge } from "../../../core/entities/bridge/bridge";
import { BridgeService } from "../../../core/entities/bridge/bridge.service";
import { CompanyClockingEventIndicationsService } from "../../../core/entities/company-clocking-event-indications/company-clocking-event-indications.service";
import { Employee } from "../../../core/entities/employee/employee";
import { EmployeeDto } from "../../../core/entities/employee/employee-dto";
import { EmployeeService } from "../../../core/entities/employee/employee.service";
import { FocusService } from "../../../shared/focus/focus.service";
import { FieldCustomizationService } from "../../../shared/services/field-customization.service";
import { removeErrorFromField, startDateIsBeforeEndDateValidator } from "../../../shared/utils/date-validator";
import { isValidFilter, mergeUnique } from "../../../shared/utils/utils";

const moment = moment_; // @HACK Necessary because of https://github.com/rollup/rollup/issues/670

@Component({
    templateUrl: "./bridge-form.component.html",
    providers: [
        ConfirmationService,
    ],
    selector: "cem-bridge-form",
    styleUrls: [
        './bridge-form.component.scss'
    ]
})
export class BridgeFormComponent implements OnInit, OnDestroy, AfterViewInit {

    public localeConfig: any = {};
    public currencyLocaleOptions: any = {};
    public permissions: any = {};
    public allPermissions: any = {};

    public formGroup: FormGroup;
    public loading: boolean;
    public mainPanelCollapsed = false;

    public contractTypeOptions: { label: string; value: string }[] = [
        {
            label: this.translate.instant(
                "hcm_clocking_event_foundation.general.contract_type_employee"
            ),
            value: "EMPLOYEE",
        },
        {
            label: this.translate.instant(
                "hcm_clocking_event_foundation.general.contract_type_director"
            ),
            value: "DIRECTOR",
        },
        {
            label: this.translate.instant(
                "hcm_clocking_event_foundation.general.contract_type_trainee"
            ),
            value: "TRAINEE",
        },
        {
            label: this.translate.instant(
                "hcm_clocking_event_foundation.general.contract_type_aprentice"
            ),
            value: "APRENTICE",
        },
        {
            label: this.translate.instant(
                "hcm_clocking_event_foundation.general.contract_type_third_party"
            ),
            value: "THIRD_PARTY",
        },
    ];

    public employeeLookupSuggestions: Employee[];
    public employeeLookupDisplayFields = [
        "employer.codEmp",
        "code",
        "person.nomPes",
        "employmentRelationship.contractType",
    ];
    public employeeAdvancedDisplayFields = ["employer.codEmp", "code", "person.nomPes", "employmentRelationship.contractType"];
    public employeeLookupSeparator = " - ";
    public employeeSearchFields: FormField[];
    public employeeSearchGridFields: FormField[];
    public employeeSearchGridData: Employee[];
    public employeeSearchTotalRecords: number;

    @ViewChild("customTemplate", { static: false })
    public customTemplate: TemplateRef<any>;

    @ViewChild("customFormFields", { static: false })
    public customFormFields: TemplateRef<any>;

    /*{CA:CLASS_ATTRIBUTES:START}*/
    public cacheLimitDaysBackwardScheduling: Map<string, Date> = new Map();
    @Input()
    onSubmitObservable: Observable<any>;
    @Input()
    onLockedEmployeeChanges: Observable<EmployeeDate>;
    @Input()
    lockedEmployee: EmployeeDate;

    private onClose: Subject<boolean> = new Subject();
    @Output() onCloseObservable: Observable<boolean> = this.onClose.asObservable();
    /*{CA:CLASS_ATTRIBUTES:END}*/

    private routeParams: any;
    private ngUnsubscribe = new Subject();

    constructor(
        /*{CA:INJECTIONS:START}*/
        private companyClockingEventIndicationsService: CompanyClockingEventIndicationsService,
        /*{CA:INJECTIONS:END}*/
        private bridgeService: BridgeService,
        private router: Router,
        private route: ActivatedRoute,
        private messageService: MessageService,
        private confirmationService: ConfirmationService,
        private formBuilder: FormBuilder,
        private translate: TranslateService,
        private hotkeysService: HotkeysService,
        private fieldCustomization: FieldCustomizationService,
        private focusService: FocusService,
        private employeeService: EmployeeService
    ) {
        /*{CA:CONSTRUCTOR_END:START}*/
        /*{CA:CONSTRUCTOR_END:END}*/
    }

    public ngOnInit() {
        /*{CA:ON_INIT_START:START}*/
        /*{CA:ON_INIT_START:END}*/

        this.formGroup = this.getFormGroup();
        if (this.lockedEmployee) {
            this.loadLockedEmployee(this.lockedEmployee)
            this.onSubmitObservable.pipe(
                takeUntil(this.ngUnsubscribe),
                filter(name => name === 'bridge')
            ).subscribe(() => this.onSave());
            if (this.onLockedEmployeeChanges) {
                this.onLockedEmployeeChanges
                    .pipe(
                        takeUntil(this.ngUnsubscribe),
                    ).subscribe((value: { employeeId: string; evaluationDate: Date; }) => {
                        this.formGroup = this.getFormGroup();
                        this.loadLockedEmployee(value);
                    })
            }
        } else {
            this.setHotkeys();
        }

        this.route.params
            .pipe(takeUntil(this.ngUnsubscribe))
            .subscribe((params: any) => this.onRouteParamsChange(params));
        this.route.data
            .pipe(takeUntil(this.ngUnsubscribe))
            .subscribe((data: any) => this.onRouteDataChange(data));

        this.employeeSearchFields = this.getEmployeeSearchFields();
        this.employeeSearchGridFields = this.getEmployeeSearchGridFields();

        /*{CA:ON_INIT_END:START}*/
        /*{CA:ON_INIT_END:END}*/
    }

    private loadLockedEmployee(value: { employeeId: string; evaluationDate: Date; }) {
        if (!this.formGroup.value.employee) {
            this.loading = true;
            this.employeeService.listExternalEmployee({
                filterQuery: `id eq '${value.employeeId}'`,
                displayFields: this.employeeLookupDisplayFields
            }).subscribe((dto: { contents: EmployeeDto[]; }) => {
                const bridge = new Bridge();
                bridge.originDate = moment(value.evaluationDate).toDate();
                bridge.employee = Employee.fromDto(dto.contents[0], "Bridge");
                this.updateSuggestions(bridge);
                this.formGroup.patchValue(bridge);
                this.formGroup.controls['employee'].disable();
                this.loading = false;
            });
        } else {
            this.formGroup.controls['bridge'].setValue(value.evaluationDate)
        }
    }

    public ngAfterViewInit() {
        /*{CA:ON_VIEW_INIT_START:START}*/
        /*{CA:ON_VIEW_INIT_START:END}*/

        /*{CA:ON_VIEW_INIT_END:START}*/
        /*{CA:ON_VIEW_INIT_END:END}*/
    }

    public ngOnDestroy() {
        /*{CA:ON_DESTROY_START:START}*/
        /*{CA:ON_DESTROY_START:END}*/

        this.ngUnsubscribe.unsubscribe();
        this.cacheLimitDaysBackwardScheduling.clear();

        /*{CA:ON_DESTROY_END:START}*/
        /*{CA:ON_DESTROY_END:END}*/
    }

    public setHotkeys() {
        /*{CA:SET_HOTKEYS_START:START}*/
        /*{CA:SET_HOTKEYS_START:END}*/

        this.hotkeysService.add(
            new Hotkey(
                "alt+shift+x",
                () => {
                    if (this.permissions.excluir) this.onDelete();
                    return false;
                },
                ["INPUT", "SELECT", "TEXTAREA"]
            )
        );

        this.hotkeysService.add(
            new Hotkey(
                "alt+shift+s",
                () => {
                    if (this.isNew() || this.permissions.editar) this.onSave();
                    return false;
                },
                ["INPUT", "SELECT", "TEXTAREA"]
            )
        );

        this.hotkeysService.add(
            new Hotkey(
                "alt+shift+c",
                () => {
                    this.onCancel();
                    return false;
                },
                ["INPUT", "SELECT", "TEXTAREA"]
            )
        );

        /*{CA:SET_HOTKEYS_END:START}*/
        /*{CA:SET_HOTKEYS_END:END}*/
    }

    public isNew() {
        return this.routeParams.bridge == "new";
    }

    public onRouteParamsChange(params: any) {
        /*{CA:ON_ROUTE_PARAMS_CHANGE_START:START}*/
        /*{CA:ON_ROUTE_PARAMS_CHANGE_START:END}*/
        this.routeParams = params;
        /*{CA:ON_ROUTE_PARAMS_CHANGE_END:START}*/
        /*{CA:ON_ROUTE_PARAMS_CHANGE_END:END}*/
    }

    public updateSuggestions(value: any) {
        /*{CA:ON_UPDATE_SUGGESTIONS_START:START}*/
        /*{CA:ON_UPDATE_SUGGESTIONS_START:END}*/

        if (value.employee) {
            value.employee.employmentRelationship.contractTypeLabel = this.loadLabelFromEnum(
                value.employee.employmentRelationship.contractType
            );
            value.employee.label = this.getSuggestionValue(
                [
                    "employer.codEmp",
                    "code",
                    "person.nomPes",
                    "employmentRelationship.contractTypeLabel",
                ],
                "employee",
                value,
                this.employeeLookupSeparator
            );
            if (value.employee.label.trim() === this.employeeLookupSeparator.trim()) {
                delete value.employee.label;
            }
        }

        /*{CA:ON_UPDATE_SUGGESTIONS_END:START}*/
        /*{CA:ON_UPDATE_SUGGESTIONS_END:END}*/
    }

    public getSuggestionValue(
        displayFields: string[],
        ownerEntity: string,
        value: any,
        separator: string
    ) {
        return displayFields
            .map((field) => this.getProp(value, `${ownerEntity}.${field}`))
            .reduce((acc: any, val: any) => `${acc}${separator}${val}`);
    }

    public onRouteDataChange(data: any) {
        /*{CA:ON_ROUTE_DATA_CHANGE_START:START}*/
        /*{CA:ON_ROUTE_DATA_CHANGE_START:END}*/
        const { currencySymbol, thousandsSeparator, decimalSeparator } = data.localeConfig.number;

        this.localeConfig = data.localeConfig;
        this.currencyLocaleOptions = {
            align: "left",
            prefix: currencySymbol,
            thousands: thousandsSeparator,
            decimal: decimalSeparator,
        };
        this.permissions = data.allPermissions.bridge;
        this.allPermissions = data.allPermissions;
        this.validateVisualization();
        if (this.lockedEmployee) {
            return;
        }
        else if (data.entity && data.entity[0]) {
            const canEdit = this.permissions.editar;
            if (!canEdit) this.formGroup.disable();
            const value: any = Bridge.fromDto(data.entity[0]);
            this.updateSuggestions(value);
            this.formGroup.patchValue(value);
            let limitDate = new Date();
            limitDate.setDate(
                limitDate.getDate() -
                data.entity[1].timeEvaluationDefinition.limitDaysBackwardScheduling
            );
            this.cacheLimitDaysBackwardScheduling.set(data.entity[1].company.id, limitDate);
        } else {
            this.formGroup.patchValue(new Bridge());
        }

        /*{CA:ON_ROUTE_DATA_CHANGE_END:START}*/
        /*{CA:ON_ROUTE_DATA_CHANGE_END:END}*/
    }

    public onCancel() {
        /*{CA:ON_CANCEL_START:START}*/
        /*{CA:ON_CANCEL_START:END}*/

        this.goBack();

        /*{CA:ON_CANCEL_END:START}*/
        /*{CA:ON_CANCEL_END:END}*/
    }

    public onSendFile(filesToDelete: any, filesToCommit: any[]) {
        /*{CA:ON_SEND_FILE:START}*/
        /*{CA:ON_SEND_FILE:END}*/

        const observables: any[] = [];

        if (!this.fieldCustomization.isEmpty(filesToDelete)) {
            observables.push(this.fieldCustomization.deleteFiles(filesToDelete));
        }
        if (!this.fieldCustomization.isEmpty(filesToCommit)) {
            observables.push(this.fieldCustomization.commitFiles(filesToCommit));
        }

        if (!this.fieldCustomization.isEmpty(observables)) {
            return forkJoin(observables);
        } else {
            return of({});
        }
    }

    public onSave() {
        /*{CA:ON_SAVE_START:START}*/
        /*{CA:ON_SAVE_START:END}*/

        this.loading = true;

        if (!this.formGroup.valid) {
            this.loading = false;
            return this.validateAllFormFields(this.formGroup);
        }

        const { filesToDelete, filesToCommit } = this.fieldCustomization.getFiles(this.formGroup);

        this.getSaveObservable()
            .pipe(
                takeUntil(this.ngUnsubscribe),
                catchError((err: any) => {
                    this.loading = false;
                    return throwError(err);
                })
            )
            .subscribe(() => {
                /*{CA:ON_SAVE_START_TOP:START}*/
                /*{CA:ON_SAVE_START_TOP:END}*/

                this.onSendFile(filesToDelete, filesToCommit)
                    .pipe(
                        takeUntil(this.ngUnsubscribe),
                        catchError((err: any) => {
                            /*{CA:ON_SEND_ERROR_TOP:START}*/
                            /*{CA:ON_SEND_ERROR_BOTTOM:END}*/
                            this.loading = false;
                            /*{CA:ON_SEND_ERROR_TOP:START}*/
                            /*{CA:ON_SEND_ERROR_BOTTOM:END}*/
                            return throwError(err);
                        })
                    )
                    .subscribe(() => {
                        this.addMessage(
                            "success",
                            "hcm_clocking_event_foundation.general.saved_message_title",
                            "hcm_clocking_event_foundation.general.saved_message_content"
                        );
                        if (this.lockedEmployee) {
                            this.onClose.next(true);
                            this.loading = false;
                        } else {
                            this.goBack();
                        }
                        /*{CA:ON_SAVE_START_BOTTOM:START}*/
                        /*{CA:ON_SAVE_START_BOTTOM:END}*/
                    });
            });

        /*{CA:ON_SAVE_END:START}*/
        /*{CA:ON_SAVE_END:END}*/
    }

    public onDelete() {
        /*{CA:ON_DELETE_START:START}*/
        /*{CA:ON_DELETE_START:END}*/

        this.confirmationService.confirm({
            message: this.translate.instant(
                "hcm_clocking_event_foundation.general.delete_confirmation_message"
            ),
            header: this.translate.instant(
                "hcm_clocking_event_foundation.general.delete_confirmation_title"
            ),
            accept: () =>
                this.getDeleteObservable().pipe(takeUntil(this.ngUnsubscribe)).subscribe(),
        });

        /*{CA:ON_DELETE_END:START}*/
        /*{CA:ON_DELETE_END:END}*/
    }

    public getProp(obj: any, path: any) {
        return path
            .split(".")
            .reduce(
                (result: any, prop: any) => (result[prop] === undefined ? "" : result[prop]),
                obj
            );
    }

    private loadLabelFromEnum(value: string) {
        return this.contractTypeOptions.filter((r) => r.value === value)[0].label;
    }

    private loadEnumFromLabel(value: string): Array<string> {
        value = value.toUpperCase().trim();
        if (value && value !== undefined) {
            return this.contractTypeOptions
                .filter((r) => {
                    const match = r.label.toUpperCase().match(value);
                    return match !== null && match.length !== 0;
                })
                .map((r) => r.value);
        }
        return [];
    }

    public onEmployeeLookupRequest(value: string) {
        /*{CA:ON_EMPLOYEE_LOOKUP_REQUEST_START:START}*/
        /*{CA:ON_EMPLOYEE_LOOKUP_REQUEST_START:END}*/
        const sort = this.employeeLookupDisplayFields.map((field: any) => ({ field, order: 1 }));

        const filterQuery = this.getEmployeeSearchFields()
            .filter(({ type }) => isValidFilter(type, value))
            .map(({ name, type }) => {
                if (typeof value == "number") return `${name} eq ${value}`;
                else if (type == FieldType.Date)
                    return `${name} eq '${moment(value).format("YYYY-MM-DD")}'`;
                else if (type == FieldType.Time)
                    return `${name} eq '${moment(value).format("HH:mm:ss")}'`;
                else if (type == FieldType.DateTime)
                    return `${name} eq '${moment(value).format()}'`;
                else if (type == FieldType.Enum)
                    return this.loadEnumFromLabel(value)
                        .map((enumConstant) => `${name} eq '${enumConstant}'`)
                        .join(" or ");
                else if (type == FieldType.String)
                    return `containing(upper(${name}), upper('${value.replace(/'/g, "\\'")}'))`;
                else return `${name} eq '${value}'`;
            })
            .filter((filter) => filter !== "")
            .join(" or ");

        const displayFields = mergeUnique(
            this.employeeLookupDisplayFields,
            this.employeeAdvancedDisplayFields
        ).concat("hireDate", "dismissalDate");
        this.employeeService
            .listExternalEmployee({ sort, filterQuery, displayFields })
            .pipe(takeUntil(this.ngUnsubscribe))
            .pipe(
                map((list: any) => ({
                    ...list,
                    contents: list.contents.map((item: any) => ({
                        ...item,
                        label: this.employeeLookupDisplayFields
                            .map((field) => {
                                if (field.indexOf(".") > 0) item[field] = this.getProp(item, field);
                                if (field === "employmentRelationship.contractType") {
                                    item[field] = this.loadLabelFromEnum(item[field]);
                                }
                                return item[field];
                            })
                            .join(this.employeeLookupSeparator),
                    })),
                }))
            )
            .subscribe((list: any) => {
                this.employeeLookupSuggestions = list.contents;
            });

        /*{CA:ON_EMPLOYEE_LOOKUP_REQUEST_END:START}*/
        /*{CA:ON_EMPLOYEE_LOOKUP_REQUEST_END:END}*/
    }

    public onEmployeeSearchRequest(event: any) {
        /*{CA:ON_EMPLOYEE_SEARCH_REQUEST_START:START}*/
        /*{CA:ON_EMPLOYEE_SEARCH_REQUEST_START:END}*/

        const { first, rows, multiSortMeta, filterData } = event;
        const page = first / rows;
        const sort = multiSortMeta;
        const filterQuery = this.getEmployeeSearchFields()
            .filter(({ name }) => filterData[name] != undefined && filterData[name] != "")
            .map(({ name, type }) => {
                const value = filterData[name];

                if (typeof value == "number") return `${name} eq ${value}`;
                else if (type == FieldType.Date)
                    return `${name} eq '${moment(value).format("YYYY-MM-DD")}'`;
                else if (type == FieldType.Time)
                    return `${name} eq '${moment(value).format("HH:mm:ss")}'`;
                else if (type == FieldType.DateTime)
                    return `${name} eq '${moment(value).format()}'`;
                else if (type == FieldType.Enum) return `${name} eq '${value}'`;
                else if (type == FieldType.String)
                    return `containing(upper(${name}), upper('${value.replace(/'/g, "\\'")}'))`;
                else return `${name} eq '${value}'`;
            })
            .join(" and ");

        const displayFields = mergeUnique(
            this.employeeLookupDisplayFields,
            this.employeeAdvancedDisplayFields
        ).concat("hireDate", "dismissalDate");
        this.employeeService
            .listExternalEmployee({ page, sort, filterQuery, displayFields })
            .pipe(takeUntil(this.ngUnsubscribe))
            .pipe(
                map((list: any) => ({
                    ...list,
                    contents: list.contents.map((item: any) => ({
                        ...item,
                        label: this.employeeLookupDisplayFields
                            .map((field) => {  // NOSONAR
                                if (field.indexOf(".") > 0) item[field] = this.getProp(item, field); // NOSONAR
                                if (field === "employmentRelationship.contractType") { // NOSONAR
                                    item[field] = this.loadLabelFromEnum(item[field]); // NOSONAR
                                } // NOSONAR
                                return item[field]; // NOSONAR
                            }) // NOSONAR
                            .join(this.employeeLookupSeparator), // NOSONAR
                    })), // NOSONAR
                })) // NOSONAR
            ) // NOSONAR
            .subscribe((list: any) => { // NOSONAR
                this.employeeSearchGridData = list.contents; // NOSONAR
                this.employeeSearchTotalRecords = list.totalElements; // NOSONAR
            }); // NOSONAR

        /*{CA:ON_EMPLOYEE_SEARCH_REQUEST_END:START}*/
        /*{CA:ON_EMPLOYEE_SEARCH_REQUEST_END:END}*/
    }

    private getFormGroup() {
        const formGroup = this.formBuilder.group(
            {
                id: [{ value: undefined, disabled: true }, Validators.compose([])],
                employee: [
                    { value: undefined, disabled: false },
                    Validators.compose([Validators.required]),
                ],
                originDate: [
                    { value: undefined, disabled: false },
                    Validators.compose([Validators.required]),
                ],
                destinationDate: [
                    { value: undefined, disabled: false },
                    Validators.compose([Validators.required]),
                ],
                createdBy: [{ value: undefined, disabled: true }, Validators.compose([])],
                createdDate: [{ value: undefined, disabled: true }, Validators.compose([])],
                lastModifiedBy: [{ value: undefined, disabled: true }, Validators.compose([])],
                lastModifiedDate: [{ value: undefined, disabled: true }, Validators.compose([])],
                custom: [],
            },
            {
                validators: [
                    this.validLimitDaysBackwardScheduling(),
                    this.startDateIsBeforeEndDate(),
                ],
            }
        );

        formGroup.controls.employee.valueChanges.subscribe((employee) => {
            if (employee) {
                let limitDaysBackwardScheduling = this.cacheLimitDaysBackwardScheduling.get(
                    employee.employer.id
                );
                if (limitDaysBackwardScheduling === undefined) {
                    this.companyClockingEventIndicationsService
                        .get(employee.employer.id)
                        .pipe(
                            catchError((error) => {
                                formGroup.controls.employee.setErrors({
                                    hasNotCompanyClockingEventIndications: { value: error },
                                });
                                return throwError(error);
                            })
                        )
                        .subscribe((companyClockingEventIndications: any) => {
                            let limitDate = new Date();
                            limitDate.setDate(
                                limitDate.getDate() -
                                companyClockingEventIndications.timeEvaluationDefinition
                                    .limitDaysBackwardScheduling
                            );
                            this.cacheLimitDaysBackwardScheduling.set(
                                companyClockingEventIndications.id,
                                limitDate
                            );
                        });
                }
            }
        });

        return formGroup;
    }

    private getEmployeeSearchFields() {
        const searchFields = [
            new FormField({
                name: "employer.codEmp",
                label: this.translate.instant(
                    "hcm_clocking_event_foundation.general.employee_employer_cod_emp"
                ),
                type: FieldType.Integer,
            }),
            new FormField({
                name: "code",
                label: this.translate.instant(
                    "hcm_clocking_event_foundation.general.employee_code"
                ),
                type: FieldType.Integer,
            }),
            new FormField({
                name: "person.nomPes",
                label: this.translate.instant(
                    "hcm_clocking_event_foundation.general.employee_person_nom_pes"
                ),
                type: FieldType.String,
            }),
            new FormField({
                name: "employmentRelationship.contractType",
                label: this.translate.instant(
                    "hcm_clocking_event_foundation.general.employee_employment_relationship_contract_type"
                ),
                type: FieldType.Enum,
                options: this.contractTypeOptions,
            }),
        ];
        /*{CA:GET_EMPLOYEE_SEARCH_FIELDS:START}*/
        /*{CA:GET_EMPLOYEE_SEARCH_FIELDS:END}*/

        return searchFields;
    }

    private getEmployeeSearchGridFields() {
        const searchGridFields = [
            new FormField({
                name: "employer.codEmp",
                label: this.translate.instant("hcm_clocking_event_foundation.general.employee_employer_cod_emp"),
                type: FieldType.Integer,
            }),
            new FormField({
                name: "code",
                label: this.translate.instant("hcm_clocking_event_foundation.general.employee_code"),
                type: FieldType.Integer,
            }),
            new FormField({
                name: "person.nomPes",
                label: this.translate.instant("hcm_clocking_event_foundation.general.employee_person_nom_pes"),
                type: FieldType.String,
            }),
            new FormField({
                name: "employmentRelationship.contractType",
                label: this.translate.instant("hcm_clocking_event_foundation.general.employee_employment_relationship_contract_type"),
                type: FieldType.Enum,
                options: this.contractTypeOptions
            }),
        ];

        /*{CA:GET_EMPLOYEE_SEARCH_GRID_FIELDS:START}*/
        /*{CA:GET_EMPLOYEE_SEARCH_GRID_FIELDS:END}*/

        return searchGridFields;
    }

    private goBack() {
        const previousRoute = "../";
        this.router.navigate([previousRoute], { relativeTo: this.route.parent });
    }

    private validateAllFormFields(formGroup: FormGroup) {
        Object.keys(formGroup.controls).forEach((field) => {
            const control = formGroup.get(field);
            if (control instanceof FormControl) control.markAsDirty({ onlySelf: true });
            else if (control instanceof FormGroup) this.validateAllFormFields(control);
        });

        this.focusService.focusFirstInvalidInput();
        /*{CA:FOCUS_FIRST_INVALID:START}*/
        /*{CA:FOCUS_FIRST_INVALID:END}*/
    }

    private getSaveObservable() {
        this.formGroup.controls['employee'].enable();
        const { value } = this.formGroup;
        const bridgeDto = Bridge.toDto(value);

        let observable;

        if (this.isNew() || (this.lockedEmployee && !value.id)) {
            observable = this.bridgeService.insert(bridgeDto);
        } else {
            const id = this.routeParams.bridge;
            observable = this.bridgeService.update(id, bridgeDto);
        }

        return observable;
    }

    private getDeleteObservable() {
        return this.bridgeService.delete(this.routeParams.bridge).pipe(
            tap(() => { // NOSONAR
                this.addMessage( // NOSONAR
                    "success", // NOSONAR
                    "hcm_clocking_event_foundation.general.deleted_message_title", // NOSONAR
                    "hcm_clocking_event_foundation.general.deleted_message_content" // NOSONAR
                ); // NOSONAR
                this.goBack(); // NOSONAR
            }) // NOSONAR
        ); // NOSONAR
    } // NOSONAR

    private validateVisualization() {
        /*{CA:VALIDATE_VISUALIZATION_START:START}*/
        /*{CA:VALIDATE_VISUALIZATION_START:END}*/
        const toValidate = ["employee"];
        if (!toValidate.every((e) => this.allPermissions[e] && this.allPermissions[e].visualizar)) {
            this.router.navigate(["forbidden"]);
        }
        /*{CA:VALIDATE_VISUALIZATION_END:START}*/
        /*{CA:VALIDATE_VISUALIZATION_END:END}*/
    }

    private validLimitDaysBackwardScheduling(): ValidatorFn {
        return (control: AbstractControl): ValidationErrors | null => {
            const originDate = control.get("originDate");
            const employee = control.get("employee");
            if (employee.value && originDate.value) {
                const limitDate = this.cacheLimitDaysBackwardScheduling.get(
                    employee.value.employer.id
                );
                if (limitDate && moment(originDate.value).isBefore(limitDate, "day")) {
                    originDate.setErrors({ exceededLimit: { value: originDate.value } });
                } else {
                    removeErrorFromField(originDate, "exceededLimit");
                }
            }

            return null;
        };
    }
    private startDateIsBeforeEndDate(): ValidatorFn {
        return (control: AbstractControl): ValidationErrors | null => {
            const originDate = control.get("originDate");
            const destinationDate = control.get("destinationDate");
            startDateIsBeforeEndDateValidator(originDate, destinationDate);
            return null;
        };
    }
    /*{CA:CLASS_METHODS:END}*/

    private addMessage(severity: string, summary: string, detail: string) {
        this.messageService.add({
            severity,
            summary: this.translate.instant(summary),
            detail: this.translate.instant(detail),
        });
    }
}