import os
from datetime import datetime
from string import capwords

from colorama import Fore
from helpers import Backend, ChangelogReader, Frontend, find_changed_files
from jira_manager import JiraManager

from common import exit_message, get_env_variable, print_message
from common.getopt_helper import GetOptBuilder, Param
from common.graphql_client import GraphqlClient


class CreateSreIssue(GetOptBuilder):
    def __init__(self, argv):
        builder = super()
        builder.__init__(argv)
        self.client_graphql = GraphqlClient()

        tag_param = Param("tag", default_value=get_env_variable("CI_COMMIT_REF_SLUG"))
        repository_param = Param(
            "repository", default_value=get_env_variable("CI_PROJECT_NAME")
        )
        module_param = Param("module", default_value="")
        changelog_base_path_param = Param("changelog_base_path", default_value=".")
        sre_team_param = Param("sre_team", is_required=True)
        translations_path_param = Param("translations_path", only_verbose=True)

        builder.add_params(
            [
                tag_param,
                repository_param,
                module_param,
                changelog_base_path_param,
                sre_team_param,
                translations_path_param,
            ]
        )

        builder.build()

        self.jira_manager = JiraManager()

        self.tag = tag_param.get_param_value()
        self.module = module_param.get_param_value()
        self.changelog_base_path = changelog_base_path_param.get_param_value()
        self.sre_team = sre_team_param.get_param_value()
        self.translations_path = translations_path_param.get_param_value()

        self.changelog_reader = ChangelogReader(
            self.changelog_base_path, self.tag, self.jira_manager
        )

        repository_name = repository_param.get_param_value()

        self.repository = (
            Frontend(self.tag, repository_name)
            if "-frontend" in repository_name
            else Backend(self.tag, repository_name)
        )

    def exec(self):
        """Reads the issues from a given version in changelog and creates an 'deploy issue'"""

        changelog_issues = self.changelog_reader.read_issues()

        if not changelog_issues:
            exit_message(
                f"Nenhuma task encontrada no CHANGELOG para a tag {self.tag}",
            )

        sre_issue_type = (
            "Bug"
            if any(issue.fields.issuetype.name == "Bug" for issue in changelog_issues)
            else "Story"
        )

        sre_issue = self._create_sre_issue(
            sre_issue_type, self.repository.deploy_name_custom_field
        )

        print_message(
            f"Task criada para o SRE: {self.jira_manager.get_issue_url(sre_issue.key)} - {sre_issue.fields.summary} ({self.tag})",
            Fore.GREEN,
        )

        for issue in changelog_issues:
            if self.jira_manager.create_link(
                "Dependência entre atividades", issue, sre_issue
            ):
                print_message(
                    f"Link criado: {self.jira_manager.get_issue_url(issue.key)} - {sre_issue.key}",
                    Fore.GREEN,
                )

    def _create_sre_issue(self, sre_issue_type, deploy_name_custom_field):

        jira_issue = {
            "project": {"key": self.sre_team},
            "summary": f"Deploy {self.repository.name}",
            "issuetype": {"name": "Deploy"},
            "labels": [sre_issue_type, self.module],
            "description": self.get_description(),
            deploy_name_custom_field: [self.repository.get_service_path()],
            "customfield_11100": [{"value": "Produção - platform.senior.com.br"}],
        }

        return self.jira_manager.create_issue(jira_issue)

    def get_description(self):
        description = "*Changelog*\n"
        description += self.changelog_reader.changelog_url
        description += "\n\n"

        translations_path = (
            self.translations_path
            if self.translations_path
            else self.repository.translations_path
        )
        changed_files = find_changed_files(translations_path)
        translations = ""
        project_url = get_env_variable("CI_PROJECT_URL")

        for translation_file in changed_files:
            filename = os.path.basename(translation_file)
            if filename:
                translations += (
                    f"[{filename}|{project_url}/-/raw/{self.tag}/{translation_file}]\n"
                )

        if translations:
            description += (
                f"*Favor atualizar os arquivos de tradução:*\n{translations}\n\n"
            )

        description += self.create_merge_request_table()

        return description

    def create_merge_request_table(self):

        table_head = "h2.Merge Requests\n||Título|||Validação do review|||Aberto por|||Fechado por|||Pipeline|||Aberto em|||Fechado em||\n"
        table_data = table_head
        merge_requests = self.get_merges_between_tags()

        for merge_request in merge_requests:

            created_at = datetime.strftime(
                datetime.fromisoformat(merge_request["createdAt"]), "%d/%m/%y %H:%M"
            )
            merged_at = datetime.strftime(
                datetime.fromisoformat(merge_request["mergedAt"]), "%d/%m/%y %H:%M"
            )
            mr_title = f"*[{merge_request['title'].replace('[', '').replace(']', '')}|{merge_request['url']}]*"
            author_name = f"[{capwords(merge_request['author'].replace('.', ' '))}|https://git.senior.com.br/{merge_request['author']}]"
            closed_by_name = f"[{capwords(merge_request['mergedBy'].replace('.', ' '))}|https://git.senior.com.br/{merge_request['mergedBy']}]"
            pipeline_status = (
                "(/)" if merge_request["pipelineStatus"] == "success" else "(x)"
            )
            review = "(/)"

            if merge_request["author"] == merge_request["mergedBy"]:
                author_name += " (flag)"
                closed_by_name += " (flag)"
                review = "(x)"

            table_row = f"|{mr_title}||{review}|{author_name}|{closed_by_name}||{pipeline_status}|{created_at}|{merged_at}|\n"
            table_data += table_row

        table_subtitle = "~O ícone (flag) indica se o Merge Request foi aberto e fechado pela mesma pessoa.~\n"
        table_data += table_subtitle

        return table_data

    def get_merges_between_tags(self):

        query = """
            query($projectId: Int!){
                mergesBetweenTags(projectId: $projectId) {
                    mergeRequestsData {
                    url
                    title
                    createdAt
                    mergedAt
                    pipelineStatus
                    author
                    assignee
                    reviewer
                    mergedBy
                    }
                }
            }
        """

        params = {
            "projectId": int(get_env_variable("CI_PROJECT_ID")),
        }

        request = self.client_graphql.call(query, params)

        return request["mergesBetweenTags"]["mergeRequestsData"]
