import json
import re
from configparser import ConfigParser
from pathlib import Path

from jira_manager import JiraManager

from common import (
    exec_command_and_return,
    get_env_variable,
    print_message,
    unshallow_repo,
)


class Repository:
    def __init__(self, tag, name):
        self.tag = tag
        self.name = name
        self.deploy_name_custom_field = ""
        self.translations_path = ""

    def get_service_path(self):
        return f"{self.name}:{self.tag}"


class Backend(Repository):
    def __init__(self, tag, name):
        super().__init__(tag, name)
        self.deploy_name_custom_field = "customfield_12001"
        self.translations_path = "java/impl/src/main/resources/translation"

    def get_service_path(self):
        sdl_properties_path = Path("sdl.properties")
        if sdl_properties_path.is_file():
            with sdl_properties_path.open("r", encoding="utf-8") as properties_data:
                config_string = "[DEFAULT]\n" + properties_data.read()

            config = ConfigParser()
            config.read_string(config_string)

            app = config["DEFAULT"]["generator.app.name"]

            return f"{app}:{self.tag}"
        else:
            return f"{self.name}:{self.tag}"


class Frontend(Repository):
    def __init__(self, tag, name):
        super().__init__(tag, name)
        self.deploy_name_custom_field = "customfield_12002"
        self.translations_path = "src/locale"

    def get_service_path(self):
        package_json_path = Path("package.json")

        if package_json_path.is_file():
            with package_json_path.open("r", encoding="utf-8") as package_data:
                json_data = json.load(package_data)

            app = json_data["project"]["app"]
            domain = json_data["project"]["domain"]

            return f"{app}/{domain}/{self.name}:{self.tag}"


class ChangelogReader:
    def __init__(
        self, changelog_base_path, tag, jira_manager: JiraManager = JiraManager()
    ):
        self.jira_manager = jira_manager

        self.issue_pattern = re.compile(
            r"\*?\s*?\[#?(?P<key>\w+-\d+)\](?P<link>\(http[s]?:\/\/[^)]*\))?\s*-?\s*(?P<summary>.*)",
            re.MULTILINE,
        )

        self.changelog_path = f"{changelog_base_path}/CHANGELOG.md"
        self.tag = tag.replace("v", "").replace("-", ".")

        project_url = get_env_variable("CI_PROJECT_URL")

        self.changelog_url = (
            f"{project_url}/-/blob/{tag}{self.changelog_path.replace('.','', 1)}"
        )

    def read_issues(self):
        issues = []
        tag_section_pattern = re.compile(r"^# [0-9\.]+")
        with open(self.changelog_path, "r", encoding="utf-8") as changelog:
            line = changelog.readline()
            is_in_tag_section = False
            while line:
                if self.tag in line:
                    is_in_tag_section = True
                    line = changelog.readline()
                    continue

                started_other_section = is_in_tag_section and tag_section_pattern.match(
                    line
                )
                if started_other_section:
                    break

                if is_in_tag_section and self.issue_pattern.match(line):
                    issue_key = self._issue_key_from(line)
                    issue = self.jira_manager.get_issue(issue_key)
                    issues.append(issue)

                line = changelog.readline()

            changelog.close()

        return issues

    def _issue_key_from(self, line):
        matches = self.issue_pattern.search(line)
        return matches.group("key")


def find_changed_files(path):
    unshallow_repo()
    previous_tag_command = exec_command_and_return(
        "git tag --sort=-creatordate | head -n 2 | tail -n 1"
    )
    previous_tag = previous_tag_command.output.strip()
    tag_or_first_commit = (
        previous_tag
        if previous_tag
        else exec_command_and_return("git rev-list HEAD | tail -n 1").output
    )

    command_return = exec_command_and_return(
        f"git diff --name-only {tag_or_first_commit} | grep {path}"
    )

    print_message(f"Arquivos atualizados: \n{command_return.output}")

    return command_return.output.split("\n")
